@@ -1,12 +1,14 @@
 package Net::DNS::RR::SPF;
 
 #
-# $Id: SPF.pm 1037 2012-10-24 21:50:15Z willem $
+# $Id: SPF.pm 1145 2013-12-17 11:39:45Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1037 $)[1];
+$VERSION = (qw$LastChangedRevision: 1145 $)[1];
 
-use base Net::DNS::RR::TXT;
+
+use strict;
+use base qw(Net::DNS::RR::TXT);
 
 =head1 NAME
 
@@ -14,15 +16,16 @@ Net::DNS::RR::SPF - DNS SPF resource record
 
 =cut
 
-
-use strict;
 use integer;
 
 
 sub spfdata {
-	join '', shift->txtdata(@_);
+	return shift->char_str_list(@_) if wantarray;
+	join '', shift->char_str_list(@_);
 }
 
+sub txtdata { &spfdata; }
+
 1;
 __END__
 
@@ -32,6 +35,16 @@ __END__
     use Net::DNS;
     $rr = new Net::DNS::RR('name SPF spfdata ...');
 
+    $rr = new Net::DNS::RR( name    => 'name',
+			    type    => 'SPF',
+			    spfdata => 'single text string'
+			    );
+
+    $rr = new Net::DNS::RR( name    => 'name',
+			    type    => 'SPF',
+			    spfdata => [ 'multiple', 'strings', ... ]
+			    );
+
 =head1 DESCRIPTION
 
 Class for DNS Sender Policy Framework (SPF) resource records.
@@ -52,22 +65,28 @@ other unpredictable behaviour.
 =head2 spfdata
 
     $string = $rr->spfdata;
+    @list   = $rr->spfdata;
 
-C<spfdata> returns the policy text as a single string, regardless
-of the actual number of elements.
+    $rr->spfdata( @list );
+
+When invoked in scalar context, spfdata() returns the policy text as
+a single string, with text elements concatenated without intervening
+spaces.
+
+In a list context, spfdata() returns a list of the text elements.
 
 
 =head1 COPYRIGHT
 
 Copyright (c)2005 Olaf Kolkman, NLnet Labs.
 
-Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
-
 All rights reserved.
 
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
 
+Package template (c)2009,2012 O.M.Kolkman and R.W.Franks.
+
 
 =head1 SEE ALSO
 
